import gtk
import nautilus
import commands
import re
import os
import urllib


def normalize(path):
    path = path.replace(" ", "\\ ")
    path = path.replace("(", "\\(")
    path = path.replace(")", "\\)")
    path = path.replace("'", "\\'")
    path = path.replace("&", "\\&")
    return path


class MountImageExtension(nautilus.MenuProvider):
    def __init__(self):
        self.file_names = []


    def menu_activate_cb_m(self, menu, path, filetype, options):
        """Called when the user selects the Mount menu."""

        name = os.path.basename(path)

        mount_output = commands.getoutput("mount")
        mounted = re.search("/media/" + name + "|" + path, mount_output)
        if not mounted:
            os.system("gksudo -k 'mkdir /media/" + name + "'")
            os.system("gksudo -k 'mount " + options + " " + filetype + " " + path + " /media/" + name + "'")
            mount_output = commands.getoutput("mount")
            mounted = re.search("/media/" + name, mount_output)
            if not mounted:
                os.system("gksudo -k 'rm -rf /media/" + name + "'")

    def menu_activate_cb_u(self, menu, path):
        """Called when the user selects the Unmount menu."""

        mount_points = commands.getoutput("mount")
        name = os.path.basename(path)

        if re.search("/media/" + name, mount_points):
            os.system("gksudo -k 'umount -d /media/" + name + "'")
            os.system("gksudo -k 'rm -rf /media/" + name + "'")
        elif re.search(path, mount_points):
            os.system("gksudo -k 'umount -d " + path + "'")

    def get_file_items(self, window, files):
        """Called when the user selects a file in Nautilus."""

        # This script will only accept one file
        if len(files) != 1:
            return

        f = files[0]
        filepath = normalize(urllib.unquote(f.get_uri()[7:]))

        # Get image format, set mount options
        # Supported images with file systems: 
        # Readonly: ISO 9660,UDF,NERO IMAGE
        # Writable: NTFS,VFAT,XFS,ReiserFS,Reiser4,JFS,EXT2/3/4,Minix,HFS,HFS+

        fileoutput = commands.getoutput("file " + filepath)

        if re.search("ISO 9660|UDF", fileoutput):
            path = filepath
            filetype = "-t udf,iso9660"
            options = "-o loop,user,ro,iocharset=utf8"
        elif re.search("NTFS|ntfs", fileoutput):
            path = filepath
            filetype = "-t ntfs"
            options = "-o loop,force"
        elif re.search("VFAT|vfat|FAT|fat|FAT32|fat32|FAT16|fat16", fileoutput):
            path = filepath
            filetype = "-t vfat"
            options = "-o loop,utf8"
        elif re.search("XFS|xfs", fileoutput):
            path = filepath
            filetype = "-t xfs"
            options = "-o loop"
        elif re.search("REISERFS|reiserfs|ReiserFS", fileoutput):
            path = filepath
            filetype = "-t reiserfs"
            options = "-o loop"
        elif re.search("EXT2|ext2", fileoutput):
            path = filepath
            filetype = "-t ext2"
            options = "-o loop"
        elif re.search("EXT3|ext3", fileoutput):
            path = filepath
            filetype = "-t ext3"
            options = "-o loop"
        elif re.search("EXT4|ext4", fileoutput):
            path = filepath
            filetype = "-t ext4"
            options = "-o loop"
        elif re.search("HFS|hfs", fileoutput):
            path = filepath
            filetype = "-t hfs,hfsplus"
            options = "-o loop"
        elif re.search("data|DATA", fileoutput):
            headoutput = commands.getoutput("head -n 6 " + filepath)
            if re.search("JFS|JFS1|jfs|jfs1", headoutput):
                path = filepath
                filetype = "-t jfs"
                options = "-o loop"
            elif re.search("NERO|Nero|nero", headoutput):
                path = filepath
                filetype = "-t udf,iso9660"
                options = "-o loop,offset=307200,ro,utf8"
            elif re.search("ReIsEr40FoRmAt", headoutput):
                path = filepath
                filetype = "-t reiser4"
                options = "-o loop"
        elif re.search("Minix|minix", fileoutput):
            path = filepath
            filetype = "-t minix"
            options = "-o loop"
        else:
            return


        # Check whether we have show the mount or the unmount menu
        name = os.path.basename(path)

        # Show mount or unmount menu(s) in correct language coresponding to system locale:
        if re.search("LANG=zh_TW.UTF-8|LANG=zh_HK.UTF-8", commands.getoutput("locale")):
            mount_item_name_iso = u"\u639b\u8f09"
            mount_item_name_ro = u"\u4ee5\u53ea\u8b80\u65b9\u5f0f\u639b\u8f09"
            mount_item_name_rw = u"\u4ee5\u8b80\u5beb\u65b9\u5f0f\u639b\u8f09"
            umount_item_name = u"\u5378\u8f09"
        elif re.search("LANG=zh_CN.UTF-8|LANG=zh_SG.UTF-8", commands.getoutput("locale")):
            mount_item_name_iso = u"\u6302\u8f7d"
            mount_item_name_ro = u"\u4ee5\u53ea\u8bfb\u65b9\u5f0f\u6302\u8f7d"
            mount_item_name_rw = u"\u4ee5\u8bfb\u5199\u65b9\u5f0f\u6302\u8f7d"
            umount_item_name = u"\u5378\u8f7d"
        else:
            mount_item_name_iso = "Mount"
            mount_item_name_ro = "Mount in readonly mode"
            mount_item_name_rw = "Mount in writable mode"
            umount_item_name = "Unmount"

        if re.search("/media/" + name + "|" + path, commands.getoutput("mount")):
            unmount_item = nautilus.MenuItem("NautilusPython::unmount_image_item",
                                     umount_item_name,
                                     "Unmount image",
                                     "nautilus-mount-image")
            unmount_item.connect("activate", self.menu_activate_cb_u, path)
            return unmount_item,
        else:
            if filetype is "-t udf,iso9660":
                mount_item_ro = nautilus.MenuItem("NautilusPython::mount_image_item_iso",
                                         mount_item_name_iso,
                                         "Mount image",
                                         "nautilus-mount-image")
                mount_item_ro.connect("activate", self.menu_activate_cb_m, path, filetype, options)
	        return mount_item_ro,
            else:
                options_ro = options + ",ro"
                options_rw = options + ",rw,user,umask=000"
                mount_item_ro = nautilus.MenuItem("NautilusPython::mount_image_item_ro",
                                         mount_item_name_ro,
                                         "Mount image",
                                         "nautilus-mount-image")
                mount_item_ro.connect("activate", self.menu_activate_cb_m, path, filetype, options_ro)
                mount_item_rw = nautilus.MenuItem("NautilusPython::mount_image_item_rw",
                                         mount_item_name_rw,
                                         "Mount image",
                                         "nautilus-mount-image")
                mount_item_rw.connect("activate", self.menu_activate_cb_m, path, filetype, options_rw)
                return mount_item_ro,mount_item_rw,
